package ocamlformat-lib

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Ocamlformat_lib.Fmt_odocSource

offset is the column at which the content of the comment begins. It is used to adjust the margin.

Sourceval fmt_ast : Conf.t -> fmt_code:fmt_code -> Odoc_parser.Ast.t -> Fmt.t

source is the global source in which the locations in the AST make sense. input is the content of the doc-comment.